﻿using System;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace SplitPolygon
{
    public partial class TestForm : Form
    {
        PolygonShape polygonBeingSplit;
        LineShape splitLineShape;
        MapShapeLayer mapShapeLayer;

        public TestForm()
        {
            InitializeComponent();
        }

        private void TestForm_Load(object sender, EventArgs e)
        {
            splitLineShape = new LineShape("LineString(15 20,-10 10,0 0,-10 -10,15 -20)");
            polygonBeingSplit = new EllipseShape(new PointShape(0, 0), 20).ToPolygon();

            mapShapeLayer = new MapShapeLayer();
            MapShape mapShapeBeingSplit = new MapShape();
            mapShapeBeingSplit.Feature = new Feature(polygonBeingSplit);
            mapShapeBeingSplit.ZoomLevels.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColors.Chocolate);
            mapShapeBeingSplit.ZoomLevels.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            mapShapeLayer.MapShapes.Add("mapShapeBeingSplit", mapShapeBeingSplit);

            MapShape splittingLineMapShape = new MapShape();
            splittingLineMapShape.Feature = new Feature(splitLineShape);
            splittingLineMapShape.ZoomLevels.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColors.Black, 3, true);
            splittingLineMapShape.ZoomLevels.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            mapShapeLayer.MapShapes.Add("splittingLineMapShape", splittingLineMapShape);

            Draw(mapShapeLayer);
        }

        private void Draw(Layer layer)
        {
            layer.Open();
            GdiPlusGeoCanvas geoCanvas = new GdiPlusGeoCanvas();
            Bitmap bitmap = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            RectangleShape extent = ExtentHelper.GetDrawingExtent(new RectangleShape(-30, 30, 30, -30), pictureBox1.Width, pictureBox1.Height);
            geoCanvas.BeginDrawing(bitmap, extent, GeographyUnit.DecimalDegree);
            layer.Draw(geoCanvas, new Collection<SimpleCandidate>());
            geoCanvas.EndDrawing();
            layer.Close();

            pictureBox1.Image = bitmap;
        }

        private void btnSplit_Click(object sender, EventArgs e)
        {
            MultipolygonShape polygons = Splitter.Split(polygonBeingSplit, splitLineShape);

            MapShape resultShape1 = new MapShape();
            resultShape1.Feature = new Feature(polygons.Polygons[0]);
            resultShape1.ZoomLevels.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColors.Blue);
            resultShape1.ZoomLevels.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            if (!mapShapeLayer.MapShapes.ContainsKey("resultShape1"))
                mapShapeLayer.MapShapes.Add("resultShape1", resultShape1);

            MapShape resultShape2 = new MapShape();
            resultShape2.Feature = new Feature(polygons.Polygons[1]);
            resultShape2.ZoomLevels.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColors.Red);
            resultShape2.ZoomLevels.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            if (!mapShapeLayer.MapShapes.ContainsKey("resultShape2"))
                mapShapeLayer.MapShapes.Add("resultShape2", resultShape2);

            Draw(mapShapeLayer);
        }
    }
}
